<?php
/**********************************************************************************
 *
 * adiImage Fotogalerie
 *
 * v.0.7 - 2007-10-13
 * Copyright 2007 Peter Haunert
 *
 * http://www.netlane.de/adiImage/
 *
 ****************************************************************************
 *        based on: Weatimages 1.7.2, Copyright 2003-2007 Vladimir Nazarkin
 *                  http: *nazarkin.name/projects/weatimages/
 *                  weatimages@nazarkin.name
 ****************************************************************************
 *
 * This file is part of adiImage Fotogalerie.
 *
 * adiImage is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * $Id$
 *
/**********************************************************************************/


//Do some initializations
error_reporting(E_ALL ^ E_NOTICE);

set_include_path(get_include_path().PATH_SEPARATOR.$adiPath);


//Check for GD2 lib
if (!function_exists(imagecopyresampled)) {
	echo "Fatal error: GD2 library is not presented";
	exit;
}

//kill oneself if user trying to access resource with '/../' in path
if (strstr(urldecode($_SERVER['REQUEST_URI']), '/../')) { fatal_error('Suspicious query rejected'); }


$limit_cpu = ini_get('max_execution_time');
$limit_mem = ini_get('memory_limit');
if ($limit_cpu && ($limit_cpu < 60)) { @ini_set('max_execution_time', '60'); }
if ($limit_mem && (rtrim($limit_mem, 'M') < 64)) { @ini_set('memory_limit', '64M'); }


$adiServerPath = createServerPath($adiDir);

function createServerPath($dir) {

	$curServerPath = dirname($_SERVER['SCRIPT_NAME']);

	if ($curServerPath == "\\")
		$curServerPath = "/";

	$adiServerPath = ($curServerPath == "/" ?  $curServerPath : $curServerPath . "/" ) .  $dir . "/";
	return $adiServerPath;
}


//$_SERVER['PATH_INFO'] and $_SERVER['PHP_SELF'] variables is very unstable with different php versions and
// with different web servers - so it is better to compute them manually
if ( preg_match('|('.$_SERVER['SCRIPT_NAME'].'([^\?]*))\???|', urldecode($_SERVER['REQUEST_URI']), $matches) ) {
	//PATH_INFO is the text between SCRIPT_NAME and QUERY_STRING.
	//$_SERVER['REQUEST_URI'] is not urldecode'd by default on Apache (unlike IIS).
	$SERVER_PHP_SELF = rtrim($matches[1], '/') . '/'; //ensure that we have trailing slash
	$REQUEST_PATH = trim($matches[2], '/');//cut hampering slashes

} else {
	$SERVER_PHP_SELF = $_SERVER['SCRIPT_NAME'] . '/';
}

if (!$REQUEST_PATH) {
	if ($_GET['path']) { //compatibility mode enabled and we have some path
		$REQUEST_PATH = $_GET['path'];
		$SERVER_PHP_SELF = $_SERVER['SCRIPT_NAME'] . '/' . $REQUEST_PATH . '/';
	}
	else { //we are in the root directory
		$REQUEST_PATH = '.';
	}
}

//$REQUEST_PATH never has starting and trailing slashes
//$SERVER_PHP_SELF always has starting and trailing slashes



//ini file name
$ini_ini_filename = $adiPath.'weatimages.ini';

//read ini file if it present
if (@is_readable($ini_ini_filename)) {
	$ini = parse_ini_file($ini_ini_filename, FALSE);
}
else {
	unset($ini);
}

if (!isset($ini['adiPath'])) {
	$ini['adiPath'] = $adiPath; // filesystem
	$ini['adiServerPath'] = $adiServerPath; // serverpath
}

//process nested configs
if ($ini['enable_nested_ini'] && '.' != $REQUEST_PATH) {
	$possible_ini_places = explode('/' , $REQUEST_PATH);
	$current_search_path = '.';
	while (count($possible_ini_places)) {
		$current_search_path .= '/' . array_shift($possible_ini_places);
		$current_config = $current_search_path . '/' . $ini_ini_filename;
		if (@is_readable($current_config)) {
			$ini_override = parse_ini_file($current_config, FALSE);
			foreach ($ini_override as $key => $value) {
				$ini[$key] = $value;
			}
			unset($ini_override);
		}//if (@is_readable(
	}//while (count($possible_ini_places
	unset($possible_ini_places, $current_search_path, $current_config);
}//if ($ini['enable_nested_ini']

//set default options if it absent in ini file of file doesn't exists
if (!isset($ini['cols'])) { $ini['cols'] = 4; }
if (!isset($ini['rows'])) { $ini['rows'] = 6; }
if (!isset($ini['enable_albums_listing'])) { $ini['enable_albums_listing'] = 1; }
if (!isset($ini['enable_albums_thumbs'])) { $ini['enable_albums_thumbs'] = 1; }
if (!isset($ini['albums_thumbs_index'])) { $ini['albums_thumbs_index'] = "index"; }
if (!isset($ini['enable_index_image_hide'])) { $ini['enable_index_image_hide'] = 1; }
if (!isset($ini['albums_cols'])) { $ini['albums_cols'] = 2; }
if (!isset($ini['enable_albums_mtime_sort'])) { $ini['enable_albums_mtime_sort'] = 0; }
if (!isset($ini['enable_albums_reverse_sort'])) { $ini['enable_albums_reverse_sort'] = 0; }
if (!isset($ini['enable_files_mtime_sort'])) { $ini['enable_files_mtime_sort'] = 0; }
if (!isset($ini['enable_files_reverse_sort'])) { $ini['enable_files_reverse_sort'] = 0; }
if (!isset($ini['albums'])) { $ini['albums'] = 50; }
if (!isset($ini['enable_additional_listing'])) { $ini['enable_additional_listing'] = 1; }
if (!isset($ini['additionals'])) { $ini['additionals'] = 20; }
if (!isset($ini['enable_album_zip'])) { $ini['enable_album_zip'] = 0; }
if (!isset($ini['enable_new_window'])) { $ini['enable_new_window'] = 0; }
if (!isset($ini['enable_size_lookup'])) { $ini['enable_size_lookup'] = 1; }
if (!isset($ini['enable_description'])) { $ini['enable_description'] = 1; }
if (!isset($ini['enable_exif'])) { $ini['enable_exif'] = 1; }
if (!isset($ini['enable_exif_date'])) { $ini['enable_exif_date'] = 0; }
if (!isset($ini['enable_cell_border'])) { $ini['enable_cell_border'] = 1; }
if (!isset($ini['enable_albums_edging'])) { $ini['enable_albums_edging'] = -1; }
if (!isset($ini['description_filename'])) { $ini['description_filename'] = 'desc.txt'; }
if (!isset($ini['description_delimiter'])) { $ini['description_delimiter'] = ' '; }
if (!isset($ini['enable_omit_ext'])) { $ini['enable_omit_ext'] = 1; }
if (!isset($ini['enable_filename_display'])) { $ini['enable_filename_display'] = 1; }
if (!isset($ini['enable_filename_cleaning'])) { $ini['enable_filename_cleaning'] = 1; }
if (!isset($ini['enable_html_container'])) { $ini['enable_html_container'] = 1; }
if (!isset($ini['enable_slideshow'])) { $ini['enable_slideshow'] = 1; }
if (!isset($ini['slideshow_delay'])) { $ini['slideshow_delay'] = 3; }
if (!isset($ini['charset'])) { $ini['charset'] = 'windows-1251'; }
if (!isset($ini['jpeg_quality'])) { $ini['jpeg_quality'] = 80; }
if (!isset($ini['hidden_prefix'])) { $ini['hidden_prefix'] = '.'; }
if (!isset($ini['enable_custom_container'])) { $ini['enable_custom_container'] = 0; }
if (!isset($ini['cache_dir'])) { $ini['cache_dir'] = "weatimages-cache"; }
if (!isset($ini['external_css'])) { $ini['external_css'] = ''; }
if (!isset($ini['langpack'])) { $ini['langpack'] = ''; }
if (!isset($ini['enable_compatibility_mode'])) { $ini['enable_compatibility_mode'] = 0; }

if (!isset($ini['thumb_size'])) { $ini['thumb_size'] = 150; }
if (!isset($ini['enable_thumbs_cache'])) { $ini['enable_thumbs_cache'] = 1; }

if (!isset($ini['enable_resize'])) { $ini['enable_resize'] = 1; }
if (!isset($ini['resize_size'])) { $ini['resize_size'] = 600; }
if (!isset($ini['upload_size'])) { $ini['upload_size'] = 1000; }
if (!isset($ini['upload_quality'])) { $ini['upload_quality'] = 100; }

if (!isset($ini['enable_smart_resize'])) { $ini['enable_smart_resize'] = 1; }
if (!isset($ini['default_size_resize'])) { $ini['default_size_resize'] = 1; }
if (!isset($ini['enable_resize_cache'])) { $ini['enable_resize_cache'] = 1; }

if (!isset($ini['enable_admin'])) { $ini['enable_admin'] = 1; }


//proccess options dependencies
// if 'enable_resize' disabled, we should to disable remaining resizing options
if (!$ini['enable_resize']) {
	$ini['resize_size'] = 0;
	$ini['enable_smart_resize'] = 0;
	$ini['default_size_resize'] = 0;
	$ini['enable_resize_cache'] = 0;
}
//'enable_smart_resize' feature requires 'enable_size_lookup' option enabled
if (!$ini['enable_size_lookup']) {
	$ini['enable_smart_resize'] = 0;
}
//'enable_omit_ext' feature is reasonable only when descriptions enabled
if (!$ini['enable_description']) {
	$ini['enable_omit_ext'] = 0;
}
//turn off exif lookup if exif module not found
if ($ini['enable_exif'] && !extension_loaded('exif')) {
	$ini['enable_exif'] = 0;
	$ini['enable_exif_date'] = 0;
}
if (!$ini['enable_html_container']) {
	$ini['enable_slideshow'] = 0;
}

//advanced options which aren't included in ini file
//Assume that thumbnail creation supported for next file extensions
$ini['thumbnailable_exts'] = 'jpg|jpeg|gif|png';
//Always ignore files with next extensions
$ini['ignore_exts'] = 'php|css|htm|html|sem';
//Array containing list of filenames which should be omitted during directory loading. Wildcards aren't allowed.
$ini['ignore_files'] = array('.', '..', $ini_ini_filename, $ini['description_filename'], $ini['cache_dir'], basename($_SERVER['SCRIPT_NAME']), '.htaccess', '.htpasswd', 'Thumbs.db', 'error_log', 'access_log', 'cgi-bin', '_notes');
//EXIF comment strings which must be not shown.
$ini['exif_comment_ignore_strings'] = array('MINOLTA DIGITAL CAMERA', 'KONICA MINOLTA DIGITAL CAMERA', 'OLYMPUS DIGITAL CAMERA');
//Formatting of the timestamp taken from EXIF tags
$ini['exif_date_format'] = 'd.m.Y H:i';

// constants
$ini['descs_prefix'] = "dsc_"; // field-prefix for descriptions


session_start();

$login = !empty($_REQUEST["login"]);
if ($ini['enable_admin']) {
	if ($login) {

		if (!isset($admin_username)) { // should be set in index.php
			$admin_username="admin";
			$admin_password="admin";
		}
		if (!empty($_REQUEST["admin"])) {
			if ($admin_username==$_REQUEST["admin"] && $admin_password==$_REQUEST["pwd"] ) {
				$isLoggedIn = true;
			   $_SESSION["adiImageSess"] = 1;
			} else {
				unset($_SESSION["adiImageSess"]);
				$isLoggedIn = false;
				$login = false;
			}
		}
	} else {
		$isLoggedIn = !empty($_SESSION["adiImageSess"]);
	}
}

require_once "init_lang.php";
require_once "guiFunctions.php";
require_once "imageFunctions.php";
require_once "fileFunctions.php";


// what are we up to ??
$menuid = $_REQUEST['action'];

switch ($menuid) {

	case 'thumb':
			//expected that in $REQUEST_PATH variable relative path to source image stored
			resize_manager($REQUEST_PATH, $ini['thumb_size'], '.thumb_' . $ini['thumb_size'] . '_' . $ini['jpeg_quality'] . '.cache', $ini['enable_thumbs_cache']);
		break;

	case 'resize':
			//expected that in $REQUEST_PATH variable relative path to source image stored
			if (!$ini['enable_resize']) { fatal_error("Fatal error: operation not permitted"); }
			resize_manager($REQUEST_PATH, $ini['resize_size'], '.resize_' . $ini['resize_size'] . '_' . $ini['jpeg_quality'] . '.cache', $ini['enable_resize_cache']);
		break;

	case 'noimage':
			$image = imagecreatetruecolor($ini['thumb_size'], floor(0.75*$ini['thumb_size']));
			$color_bg = imagecolorallocate($image, 255, 255, 255);
			imagefill($image, 0, 0, $color_bg);
			header("Content-Type: image/png");
			imagepng($image);
			imagedestroy($image);
		break;

	case 'big':
			//show big image inside html page
			page_big();
		break;

	case 'zip':
			if ($ini['enable_album_zip'] && is_dir($REQUEST_PATH)) {
				zip_directory($REQUEST_PATH);
			}
			else {
				fatal_error('Operation not permitted');
			}
		break;

	case 'add':
			processUpload();
		break;

	case 'rem':
			if ($isLoggedIn) {
				$filename = htmlspecialchars($_REQUEST['pic']);
				$file = getCurrentPath(false)."/".$filename;
				if (isPic($file)) {
					if (@unlink($file))
						fatal_error("Successfully removed ".$filename);
					else
						fatal_error("ERROR: Could not remove ".$filename);
				} else if (is_dir($file)) {
					if (@rmdir($file))
						fatal_error("Successfully removed ".$filename);
					else
						fatal_error("ERROR: Could not remove '".$filename."'<br><br>Possible reasons:<br> - album is not empty.<br> - album is write-protected.");
				} else {
					fatal_error('Delete '.$filename.'. Operation not permitted ');
				}
			}
		break;

	case 'descs':
			editDescsForm();
		break;

	case "savedescs":
			saveDescriptions();
		break;

	case '':
	default:
			displayAlbum();
}


exit();


?>

