<?php
/**********************************************************************************
 *
 * adiImage Fotogalerie
 *
 * v.0.7 - 2007-10-13
 * Copyright 2007 Peter Haunert
 *
 * http://www.netlane.de/adiImage/
 *
 ****************************************************************************
 *        based on: Weatimages 1.7.2, Copyright 2003-2007 Vladimir Nazarkin
 *                  http: *nazarkin.name/projects/weatimages/
 *                  weatimages@nazarkin.name
 ****************************************************************************
 *
 * This file is part of adiImage Fotogalerie.
 *
 * adiImage is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * $Id$
 *
/**********************************************************************************/





/**
 * calculate size of image after resizing
 * this will only reduce, not enlarge!
 */
function calc_size_after_resize($imagesize, $max_size) {
	//$imagesize - array containing size of source image: $imagesize[0] - width, $imagesize[1] - height
	//$max_size - maximal dimension of resulting image in pixels, or in percents
	//returns $resizedsize - size of image after resizing: $resizesize[0] - width, $resizesize[1] - height
	if (0 == $imagesize[0] || 0 == $imagesize[1]) { return array(0, 0); }
	if (preg_match('/([\.\d]+)\%$/', $max_size, $matches)) {
		$scale_factor = $matches[1] / 100;
		// dont scale up
		if($scale_factor > 1)
			return $imagesize;

		$resizedsize[0] = ceil( $imagesize[0] * $scale_factor );
		$resizedsize[1] = ceil( $imagesize[1] * $scale_factor );
		return $resizedsize;
	}

	// dont scale up
	if ($imagesize[0] <= $max_size && $imagesize[1] <= $max_size)
		return $imagesize;

	if ($imagesize[0] > $imagesize[1]) { //if width > height
		$resizedsize[0] = $max_size; //width
		$resizedsize[1] = ceil( ($imagesize[1] / $imagesize[0]) * $resizedsize[0] ); //calculate height
	}
	else { //if height > width
		$resizedsize[1] = $max_size;
		$resizedsize[0] = ceil( ($imagesize[0] / $imagesize[1]) * $resizedsize[1] );
	}
	return $resizedsize;
}//function calc_size_after_resize(





//immediate image resizing function
function resize_image ($image_input, $image_output, $max_size, $jpeg_quality) {
	//$image_input - path to file with original image
	//$image_output - path to file in which this function must to save resized image. if it empty then output image directly to browser
	//$max_size - maximal dimension of resulting image in pixels
	//$jpeg_quality - quality of resulting jpeg image
	$imagesize = getimagesize($image_input);
	if (!$imagesize) { fatal_error("Fatal error: getimagesize($image_input) failed"); }
	if (0 == $imagesize[0] || 0 == $imagesize[1]) { fatal_error("Fatal error: zero size happened"); }
	$resizedsize = calc_size_after_resize($imagesize, $max_size);
	//create image from source file
	switch ($imagesize[2]) { //find out a format of source image
		case 1: //gif
			$image = imagecreatefromgif($image_input);
			break;
		case 2: //jpg
			$image = imagecreatefromjpeg($image_input);
			break;
		case 3: //png
			$image = imagecreatefrompng($image_input);
			break;
		default: //unknown image type
			fatal_error("Fatal error: unsupported file type");
	}
	if (!$image) { fatal_error("Fatal error: cannot create image from source"); }
	$resized = imagecreatetruecolor($resizedsize[0], $resizedsize[1]);
	imagecopyresampled($resized, $image, 0, 0, 0, 0, $resizedsize[0], $resizedsize[1], $imagesize[0], $imagesize[1]);
	if (!$image_output) { //in this case we output image directly to the browser
		header('Content-Type: image/jpeg');
	}
	if(@imagejpeg($resized, $image_output, $jpeg_quality))
		return true;

	return false;
}//function resize_image (



//manages images resizing proccess and perform caching of resized images
function resize_manager($image_input, $max_size, $cache_suffix, $is_cache_enabled) {
	//$image_input - path to file with original image
	//$max_size - maximal dimension of resulting image in pixels
	//$cache_suffix - suffix used to get filename of cached thumbnail from source image filename
	//$is_cache_enabled - indicates enabled or disabled caching of resized images
	global $ini;
	if (!@is_readable($image_input)) { fatal_error("Fatal error: requested image is not readable"); }
	$is_resized_sent = FALSE;
	if ($is_cache_enabled) {
		$file_cache = $ini['cache_dir'] . '/' . $image_input . $cache_suffix;
		if (!@is_readable($file_cache)) { //thumbnail file still don't exists
			$file_cache_dir = dirname($file_cache);
			if (!@is_dir($file_cache_dir)) {
				//we need to create a copy of directories tree under cache directory
				$dirs_must_exist = array($file_cache_dir);
				while ('.' != $dirs_must_exist[count($dirs_must_exist)-1] && '/' != $dirs_must_exist[count($dirs_must_exist)-1]) {
					array_push( $dirs_must_exist, dirname($dirs_must_exist[count($dirs_must_exist)-1]) );
				}
				array_pop($dirs_must_exist);
				while (count($dirs_must_exist) > 0) {
					$tmp_dir_new = array_pop($dirs_must_exist);
					if (@is_dir($tmp_dir_new)) {
						continue;
					}
					@mkdir($tmp_dir_new, 0777);
					@chmod($tmp_dir_new, 0777);
					clearstatcache();
				}
			}
			if (@is_writable(dirname($file_cache))) {
				$file_mutex = $ini['cache_dir'] . '/' . 'weatimages-mutex';//we allow only single image resizing proccess in a time to avoid memory issues on the server
				$FP = @fopen($file_mutex, 'r');
				if (FALSE === $FP) {
					$FP = @fopen($file_mutex, 'w');
					@chmod($file_mutex, 0666);
				}
				@flock($FP, LOCK_EX);
				resize_image($image_input, $file_cache, $max_size, $ini['jpeg_quality']);
				@chmod($file_cache, 0666);
				@flock($FP, LOCK_UN);
				@fclose($FP);
			}
			clearstatcache();
		}//if (!@is_readable($file_cache)) {
		if (@is_readable($file_cache)) { //check that we successfully created thumbnail file at the previous step
			header('Content-Type: image/jpeg');
			readfile($file_cache);
			$is_resized_sent = TRUE;
		}//if (@is_readable($file_cache)) {
	 }
	if (!$is_resized_sent) { //we still don't have cached image. Let's resize image and output result to user's browser
		resize_image($image_input, '', $max_size, $ini['jpeg_quality']);
	}

}//function resize_manager()



//checks exif comment text
function is_exif_text_bad(&$exif_text) {
	global $ini;
	$exif_text = preg_replace('/^[\x00\s]+/', '', $exif_text);//cut spaces at the beginning
	$exif_text = preg_replace('/[\x00\s]+$/', '', $exif_text);//cut trailing spaces
	$search_result = array_search($exif_text, $ini['exif_comment_ignore_strings']);
	if ( false !== $search_result && NULL !== $search_result) { $exif_text = ''; }//array_search in php<4.2 returns NULL instead of false if not found
	if ( preg_match('/[\.,:\-\?\S\x00]/', $exif_text) ) { return false; } else { return true; }
}



//get image size, size after resize, resize expediency, and EXIF data
//returns array($filesize_kb, $imagesize, $resizesize, $is_resize_senseless);
function get_image_sizes($image_path) {
	global $ini;
	$exif_string = '';
	$filesize_kb = 0;
	$imagesize = array();
	if ($ini['enable_exif'] && preg_match('/\.(jpg|jpeg|tif|tiff)$/i', $image_path)){
		$exif_desc = '';
		$exif_comment = '';
		$exif_data = @exif_read_data($image_path, false, true, false);
		//try to get description from exif. it may be contained in various exif sections
		if ( is_exif_text_bad($exif_desc) && isset($exif_data['IFD0']['Title']) && extension_loaded('mbstring') ) {
			//Title given by Windows Explorer, UNICODE
			$exif_desc = @mb_convert_encoding($exif_data['IFD0']['Title'], strtoupper($ini['charset']), 'UCS-2LE');
		}
		if ( is_exif_text_bad($exif_comment) && isset($exif_data['IFD0']['Comments']) && extension_loaded('mbstring') ) {
			//Comment given by Windows Explorer, UNICODE
			$exif_comment = @mb_convert_encoding($exif_data['IFD0']['Comments'], strtoupper($ini['charset']), 'UCS-2LE');
		}
		if ( is_exif_text_bad($exif_comment) && isset($exif_data['EXIF']['UserComment']) && extension_loaded('mbstring') ) {
			//UserComment given by ACDSee, UNICODE
			$comment_field = $exif_data['EXIF']['UserComment'];
			if (preg_match('/^([^\0]+?)\0(.+)$/', $comment_field, $matches) ) {
				//now $matches[1] contains encoding of the comment (unicode most likely), $matches[2] - comment in this encoding
				if ('UNICODE' == $matches[1]) { $matches[1] = 'UCS-2LE'; }
				$exif_comment = @mb_convert_encoding($matches[2], strtoupper($ini['charset']), $matches[1]);
			}
		}
		if ( is_exif_text_bad($exif_desc) && isset($exif_data['IFD0']['ImageDescription']) ) {
			//ImageDescription given by ACDSee, non-UNICODE, international characters saved
			$exif_desc = $exif_data['IFD0']['ImageDescription'];
		}
		if ( is_exif_text_bad($exif_desc) && isset($exif_data['WINXP']['Title']) ) {
			//Title given by Windows Explorer, non-UNICODE
			$exif_desc = $exif_data['WINXP']['Title'];
		}
		if ( is_exif_text_bad($exif_comment) && isset($exif_data['WINXP']['Comments']) ) {
			//Comment given by Windows Explorer, non-UNICODE
			$exif_comment = $exif_data['WINXP']['Comments'];
		}
		if ( is_exif_text_bad($exif_comment) && isset($exif_data['COMPUTED']['UserComment']) ) {
			//UserComment given by ACDSee, autoconverted to ASCII probably with national characters loss
			$exif_comment = $exif_data['COMPUTED']['UserComment'];
		}

		if (!is_exif_text_bad($exif_desc) && !is_exif_text_bad($exif_comment)) {
			$exif_string = $exif_desc . '<br/>' . $exif_comment;
		}
		else if (!is_exif_text_bad($exif_desc)) { $exif_string = $exif_desc; }
		else if (!is_exif_text_bad($exif_comment)) { $exif_string = $exif_comment; }

		//try to get filesize from exif
		if ( isset($exif_data['FILE']['FileSize']) ) {
			$filesize_kb = $exif_data['FILE']['FileSize'];
		}
		//try to get image dimensions from exif
		if ( isset($exif_data['COMPUTED']['Width']) && isset($exif_data['COMPUTED']['Height']) ) {
			$imagesize[0] = $exif_data['COMPUTED']['Width'];
			$imagesize[1] = $exif_data['COMPUTED']['Height'];
		}
		//try to get date picture taken
		if ( $ini['enable_exif_date'] && isset($exif_data['EXIF']['DateTimeOriginal'])
			&& preg_match('/^(\d\d\d\d):(\d\d):(\d\d) (\d\d):(\d\d):(\d\d)$/', $exif_data['EXIF']['DateTimeOriginal'], $matches) ) {
			$unixtime = mktime($matches[4], $matches[5], $matches[6], $matches[2], $matches[3], $matches[1]);
			if ($unixtime > 0) {
				$exif_string = (is_exif_text_bad($exif_string)?'':$exif_string.'<br/>') . date($ini['exif_date_format'], $unixtime);
			}
		}
	}
	if (!$filesize_kb) {
		$filesize_kb = @filesize($image_path);
	}
	$filesize_kb = ceil($filesize_kb / 1024);
	if (0 == $imagesize[0] || 0 == $imagesize[1]) {
		$imagesize = @getimagesize($image_path);
	}
	$resizesize = calc_size_after_resize($imagesize, $ini['resize_size']);
	$is_resize_senseless = FALSE;
	if (($resizesize[0] >= $imagesize[0]) && $ini['enable_smart_resize']) {
		$is_resize_senseless = TRUE;
	}
	return array($filesize_kb, $imagesize, $resizesize, $is_resize_senseless, $exif_string);
}//function get_image_sizes(







?>